'''This module handles one-time pad encryption. This encryption method works
great for Anon+ because it allows for data of any size to be transfered without
an attacker being able to know anything about the data other than its length.
However, pad exchanges need to be done via a different encryption methods such
as GPG encryption.'''
from Crypto.Cipher import XOR
import libs.encryption.base
from libs.errors import UsageError

class Encryption(libs.encryption.base.Encryption):
    '''One-time pad encryption class.'''
    def encrypt(self, data):
        '''Encrypt data using a one-time pad.'''
        if self.pad_len < len(data):
            raise Exception('The pad is to short for this data')

        encrypted = self.pad.encrypt(data)
        self.pad_len -= len(data)

        return encrypted

    def decrypt(self, data):
        '''Decrypt data using a one-time pad.'''
        return self.encrypt(data)

    def set_pad(self, pad):
        '''Set the pad to given pad object.'''
        self.pad = XOR.new(pad)
        self.pad_len = len(pad)
