'''Tunnel to use to connect to other nodes over Tor.'''
import socks
import libs.threadmanager

class Tunnel(object):
    '''Base Tunnel class. Should be subclassed by other tunnels for
    compatibility reasons as well as ease of use.
    '''
    def __init__(self, node):
        self.node = node
        # TODO: need to get node onion address and port
    
    def connect(self):
        '''Use this tunnel to connect to `address`:`port`'''
        self.connection = Connection(self.node)

    def disconnect(self, nodeid):
        '''Disconnect from a node at `nodeid`'''
        pass
        
    def send(self, message):
        self.connection.send(message)

class Connection(object):
    '''A class to store a connection to a peer'''
    def __init__(self, node):
        self.sock = socks.socksocket()
        self.sock.setproxy(socks.PROXY_TYPE_SOCKS5, addr='localhost',
                           port=9050, rdns=True)
        # node.ip must not be unicode because of this bug in python-socks:
        #   https://code.google.com/p/httplib2/issues/detail?id=179
        self.sock.connect((str(node.ip), node.port))
        
    def send(self, message):
        broadcast('logthis', 'Sending: ' + message, level = 0)
        self.sock.send(message)

class Listener(libs.threadmanager.Thread):
    '''A class to listen for connections sent to our Tor onion address.
    Messages to our Tor onion address should be recieved on a localhost
    port unless otherwise set. We have no way to associate which friend
    this is so it must be added to the handshake somehow.
    '''
    def __init__(self, port=1337):
        super(Listener, self).__init__()
        self.sock = socket.socket()
        self.sock.bind(('localhost', port))
        self.sock.listen(2)
        self.sock.setblocking(False)

    def run(self):
        while not self._stop.isSet():
            # TODO: find out which friend this is without IP address
            connection = self.sock.accept() # TODO: this blocks!!!!!
            # TODO: convert this connection to a Connection object!!

def start():
    listener = Listener()
    listener.start()
    libs.threadmanager.register(listener)