from urllib import unquote_plus
import hashlib
import time

import libs
import libs.events
import libs.friends
import uis.web.content
import libs.encryption.rsa

class Handler(libs.events.Handler):
    '''A demonstration of how event handlers can serve pages on the localhost
    web server. This can be used easily accross the entire project.
    '''
    def __init__(self):
        self.posts = [
            Post('The Devs',
                 'def688dad01ddad6cdbcc9e362c9231f47dbffc4b9629e5ca844bb52731105ba',
                 'Welcome to Anon+.\nJust a warning, this system is not completely secure but it is probably better than the alternatives.'
            )
        ]

    def got_message(self, packet):
        contents = packet.message.split(',', 2)
        name = contents[0]
        sha256 = contents[1]
        post = contents[2]

        self.posts.insert(0, Post(name, sha256, post))

    def web_ui_request(self, path, connection):
        request_array = [['/','==','root'],['/global.css','==','global_css'], ['/friends.html', '==', 'friends_html'], ['/settings.html', '==', 'settings_html'], ['/shutdown.html','==','shutdown_html'], ['/make_post.cgi?',"<-",'make_post_cgi'], ['/forward.cgi?',"<-",'forward_cgi'], ['/reply.cgi?',"<-",'reply_cgi'], ['/add_friend.cgi?',"<-",'add_friend_cgi'], ['/keys.html', '==','keys_html']]
        for request in request_array:
            if request[1] == "==":
                if path ==request[0]:
                    getattr(self, request[2],self.__defaultFailure)(path, connection)
            elif request[1] == "<-":
                if path.startswith(request[0]):
                    getattr(self, request[2],self.__defaultFailure)(path, connection)

    def __defaultFailure(self, path, connection):	    
            connection.wfile.write('unrecognized request')

    def make_post_cgi(self, path, connection):
            parameters = unquote_plus(path.split('?')[-1]).split('&')
            for parameter in parameters:
                item = parameter.split('=')
                if item[0] == 'post':
                    post_contents = item[1] + ' (%s)' % time.ctime()
            for friend in libs.globals['friends'].values():
                friend.send_message(','.join((
                        libs.config['username'],
                        hashlib.sha256(post_contents).hexdigest(),
                        post_contents)))
            self.posts.insert(0, Post(
                    libs.config['username'],
                    hashlib.sha256(post_contents).hexdigest(),
                    post_contents))

            connection.send_response(301)
            connection.send_header('Location', 'http://localhost:7777/')
            connection.end_headers()
            connection.wfile.write('Redirecting')

    def shutdown_html(self, path, connection):
            connection.send_response(200)
            connection.send_header('Content-type',	'text/html')
            connection.end_headers()
            print('Got shutdown request from web server')
            connection.wfile.write(uis.web.content.template.format(
                    title = 'Shutting down',
                    pagetitle = 'Shutdown',
                    main = 'We are quitting now. Threads are being killed.',
                    sidecontent = 'Goodbye :)'
            ))
            libs.globals['running'] = False
            libs.threadmanager.killall()
            libs.threadmanager.close_sockets()

    def settings_html(self, path, connection):
            connection.send_response(200)
            connection.send_header('Content-type',	'text/html')
            connection.end_headers()
            connection.wfile.write('Connections page')


    def forward_cgi(self, path, connection):
            connection.send_response(200)
            connection.send_header('Content-type',	'text/html')
            connection.end_headers()
            print('Got shutdown request from web server')
            connection.wfile.write(uis.web.content.template.format(
                    title = 'Shutting down',
                    pagetitle = 'Shutdown',
                    main = 'We are quitting now. Threads are being killed.',
                    sidecontent = 'Goodbye :)'
            ))
            libs.globals['running'] = False
            libs.threadmanager.killall()
            libs.threadmanager.close_sockets()

    def forward_cgi(self, path, connection):
            parameters = unquote_plus(path.split('?')[-1])
            post_hash = parameters.split('=')[-1]

            for post in self.posts:
                print('Post: %s; FWD: %s' % (post.hash, post_hash))
                if post.hash == post_hash:
                    for friend in libs.globals['friends'].values():
                        friend.send_message(','.join((
                                post.name,
                                post.sha256,
                                post.body
                        )))
                    self.posts.insert(0, post)
                    break

            connection.send_response(301)
            connection.send_header('Location', 'http://localhost:7777/')
            connection.end_headers()
            connection.wfile.write('Redirecting')

    def reply_cgi(self, path, connection):
            connection.send_response(301)
            connection.send_header('Location', 'http://localhost:7777/')
            connection.end_headers()
            connection.wfile.write('Redirecting')

    def friends_html(self, path, connection):
            connection.send_response(200)
            connection.send_header('Content-type', 'text/html')
            connection.end_headers()
            connection.wfile.write(uis.web.content.template.format(
                    title = 'Friend management',
                    pagetitle = 'Friends',
                    main = uis.web.content.friend_page.format(
                            key = libs.encryption.rsa.export_key(
                                libs.config['nodekey']
                            )
                        ),
                    sidecontent = self.__friends2html()
            ))


    def add_friend_cgi(self, path, connection):
            parameters = unquote_plus(path.split('?')[-1]).split('&')
            keydata = ''
            tunnel = ''
            port = 1337
            name = ''
            ip = ''

            # Parse parameters
            for parameter in parameters:
                item = parameter.split('=')
                if item[0] == 'key':
                    keydata = item[1].replace('\\r', '').replace('\\n', '\n')
                    result = libs.encryption.rsa.import_key(keydata)
                elif item[0] == 'name':
                    name = item[1]
                elif item[0] == 'ip':
                    ip = item[1]
                elif item[0] == 'tunnel':
                    tunnel = item[1]

            # Add the friend object
            libs.friends.add_friend(result, ip, port, name, tunnel)

            # Notify the user of success
            connection.send_response(200)
            connection.send_header('Content-type', 'text/html')
            connection.wfile.write(uis.web.content.template.format(
                    title = 'Friend added',
                    pagetitle = 'Friend added',
                    main = 'Your friend has been added.<br />' +
                        '<br />Key: %s' % result,
                    sidecontent = self.__friends2html()
            ))


    def keys_html(self, path, connection):
            connection.send_response(200)
            connection.send_header('Content-type', 'text/html')
            connection.end_headers()
            connection.wfile.write(uis.web.content.template.format(
                    title = 'Key management',
                    pagetitle = 'Key management',
                    main = uis.web.content.key_form,
                    sidecontent = self.__friends2html()
            ))

    def root(self,path,connection):
            connection.send_response(200)
            connection.send_header('Content-type',	'text/html')
            connection.end_headers()
            connection.wfile.write(uis.web.content.template.format(
                    title = 'Anon+ News Feed',
                    pagetitle = 'Anon+ News Feed',
                    main = uis.web.content.post_box + self.__generate_news_feed(),
                    sidecontent = self.__friends2html()
            ))

    def global_css(self,path,connection):
            connection.send_response(200)
            connection.send_header('Content-type',	'text/css')
            connection.end_headers()
            connection.wfile.write(uis.web.content.globalcss)

    def __friends2html(self):
        '''Convert our friends list into some nice HTML'''
        return uis.web.content.friends_box.format(
                friends = 'Our friend detector tells me you have no friends!'
        )

    def __generate_news_feed(self):
        '''Return the post elements in HTML'''
        content = ''
        for post in self.posts:
            if post.type == 'mention':
                content += uis.web.content.mention.format(
                        user = post.name,
                        body = post.body,
                        hash = post.hash
                )
            elif post.type == 'self':
                content += uis.web.content.self_post.format(
                        user = post.name,
                        body = post.body,
                        hash = post.hash
                )
            else: # normal psot
                content += uis.web.content.post.format(
                        user = post.name,
                        body = post.body,
                        hash = post.hash
                )
        return content

class Post(object):
    def __init__(self, name, sha256, post):
        self.name = name
        self.sha256 = sha256
        self.hash = sha256[0:10]
        self.body = post

        if name == libs.config['username']:
            self.type = 'self'
        elif '@'+libs.config['username'] in self.body:
            self.type = 'mention'
        else:
            self.type = 'normal'

        # Sanity checks
        if not self.check_hash():
            self.body = 'This post failed the hash check.'
        self.sanatize()

    def check_hash(self):
        '''Check if the provided hash matches the informationw we were given.
        Return True if it does, otherwise False.
        '''
        return hashlib.sha256(self.body).hexdigest()[0:10] == self.hash

    def sanatize(self):
        '''Check the post options to make sure they do not contain anything
        that might act as a security risk or compromise an identity.'''
        self.name = self.name.replace('<', '&lt;')
        self.body = self.body.replace('<', '&lt;')
        self.name = self.name.replace('>', '&gt;')
        self.body = self.body.replace('>', '&gt;')
        self.name = self.name.replace('"', '&quot;')
        self.body = self.body.replace('"', '&quot;')

        # Features such as auto-linking and line breaks
        self.body = self.body.replace('\n', '<br />')
